@@ -1,39 +1,35 @@
 package Mail::Mbox::MessageParser;
 
+no strict;
+
+@ISA = qw(Exporter);
+
 use strict;
 use Carp;
-use FileHandle::Unget;
-use File::Spec;
-use File::Temp;
+use FileHandle;
+
 sub dprint;
 
 use Mail::Mbox::MessageParser::Perl;
 use Mail::Mbox::MessageParser::Grep;
 use Mail::Mbox::MessageParser::Cache;
 
-use vars qw( @ISA $VERSION $DEBUG $MAX_TESTCHAR_BUFFER_SIZE %PROGRAMS
-             $FROM_PATTERN $UPDATING_CACHE );
+use vars qw( $VERSION $DEBUG $FROM_PATTERN $UPDATING_CACHE %PROGRAMS );
 
-@ISA = qw(Exporter);
-
-$VERSION = '1.21.0';
+$VERSION = '1.10';
 $DEBUG = 0;
 
-$MAX_TESTCHAR_BUFFER_SIZE = 1048576;
-
 %PROGRAMS = (
- 'grep' => '/usr/bin/grep',
- 'tzip' => undef,
- 'gzip' => '/sw/bin/gzip',
- 'compress' => '/sw/bin/gzip',
- 'bzip' => '/sw/bin/bzip2',
- 'bzip2' => '/sw/bin/bzip2',
+ 'grep' => '/bin/grep',
+ 'tzip' => '/home/david/bin/tzip',
+ 'gzip' => '/bin/gzip',
+ 'compress' => '/bin/gzip',
+ 'bzip' => '/usr/bin/bzip2',
+ 'bzip2' => '/usr/bin/bzip2',
 );
 
-# X-From-Line is used by Gnus, and From is used by normal Unix
-# format. Newer versions of Gnus use X-Draft-From
 $FROM_PATTERN = q/(?x)^
-    (X-Draft-From:\s|X-From-Line:\s|
+    (X-Draft-From:\s.*|X-From-Line:\s.*|
     From\s
       # Skip names, months, days
       (?> [^:]+ )
@@ -43,14 +39,7 @@ $FROM_PATTERN = q/(?x)^
       (?: \s+ (?: [A-Z]{2,3} | [+-]?\d{4} ) ){1,3}
       # smail compatibility
       (\sremote\sfrom\s.*)?
-    )/;
-
-#-------------------------------------------------------------------------------
-
-*UPDATING_CACHE = \$Mail::Mbox::MessageParser::Cache::UPDATING_CACHE;
-*SETUP_CACHE = \&Mail::Mbox::MessageParser::Cache::SETUP_CACHE;
-*CLEAR_CACHE = \&Mail::Mbox::MessageParser::Cache::CLEAR_CACHE;
-*WRITE_CACHE = \&Mail::Mbox::MessageParser::Cache::WRITE_CACHE;
+    )$/;
 
 #-------------------------------------------------------------------------------
 
@@ -73,6 +62,13 @@ sub dprint
 
 #-------------------------------------------------------------------------------
 
+*UPDATING_CACHE = \$Mail::Mbox::MessageParser::Cache::UPDATING_CACHE;
+*SETUP_CACHE = \&Mail::Mbox::MessageParser::Cache::SETUP_CACHE;
+*CLEAR_CACHE = \&Mail::Mbox::MessageParser::Cache::CLEAR_CACHE;
+*WRITE_CACHE = \&Mail::Mbox::MessageParser::Cache::WRITE_CACHE;
+
+#-------------------------------------------------------------------------------
+
 sub new
 {
   my ($proto, $options, $cache_options) = @_;
@@ -89,22 +85,13 @@ sub new
     $options->{'enable_grep'} = 0;
   }
 
-  $DEBUG = $options->{'debug'}
-    if defined $options->{'debug'};
+  my ($file_type, $need_to_close_filehandle, $error);
 
-  my ($file_type, $need_to_close_filehandle, $error, $endline);
-
-  ($options->{'file_handle'}, $file_type, $need_to_close_filehandle, $error, $endline) =
+  ($options->{'file_handle'}, $file_type, $need_to_close_filehandle, $error) =
     _PREPARE_FILE_HANDLE($options->{'file_name'}, $options->{'file_handle'});
 
-  if (defined $error &&
-    !($error eq 'Not a mailbox' && $options->{'force_processing'}))
-  {
-    # Here I assume the only error for which the filehandle was opened is
-    # "Not a mailbox"
-    close $options->{'file_handle'} if $error eq 'Not a mailbox';
-    return $error;
-  }
+  return $error unless !defined $error ||
+    ($error eq 'Not a mailbox' && $options->{'force_processing'});
 
   # Grep implementation doesn't support compression right now
   $options->{'enable_grep'} = 0 if _IS_COMPRESSED_TYPE($file_type);
@@ -163,14 +150,15 @@ sub new
 
   dprint "Instantiate mailbox parser implementation: " . ref $self;
 
+  $DEBUG = $options->{'debug'}
+    if defined $options->{'debug'};
+
   $self->_print_debug_information();
 
   $self->_read_prologue();
 
   $self->{'need_to_close_filehandle'} = $need_to_close_filehandle;
 
-  $self->{'endline'} = $endline;
-
   return $self;
 }
 
@@ -185,28 +173,15 @@ sub DESTROY
 
 #-------------------------------------------------------------------------------
 
-# Returns:
-# - a file handle to the decompressed mailbox
-# - the file type (see _GET_FILE_TYPE)
-# - a boolean indicating whether the caller needs to close the file handle
-# - an error message (or undef)
-# - the endline: "\n", "\r\n", or undef
+# Returns a file handle to the decompressed mailbox.
 sub _PREPARE_FILE_HANDLE
 {
   my $file_name = shift;
   my $file_handle = shift;
 
-  dprint "Preparing file handle";
-
   if (defined $file_handle)
   {
-    # Promote this to a FileHandle::Unget if it isn't already
-    $file_handle = new FileHandle::Unget($file_handle)
-      unless UNIVERSAL::isa($file_handle, 'FileHandle::Unget');
-
-    binmode $file_handle;
-
-    my $file_type = _GET_FILE_TYPE(\$file_handle);
+    my $file_type = _GET_FILE_TYPE($file_handle);
     dprint "Filehandle file type: $file_type";
 
     # Do decompression if we need to
@@ -215,57 +190,51 @@ sub _PREPARE_FILE_HANDLE
       my ($decompressed_file_handle,$error) =
         _DO_DECOMPRESSION($file_handle, $file_type);
 
-      return ($file_handle,$file_type,0,$error,undef)
+      return ($file_handle,$file_type,0,$error)
         unless defined $decompressed_file_handle;
 
-      return ($decompressed_file_handle,$file_type,0,"Not a mailbox",undef)
-        if _GET_FILE_TYPE(\$decompressed_file_handle) ne 'mailbox';
+      return ($decompressed_file_handle,$file_type,0,"Not a mailbox")
+        if _GET_FILE_TYPE($decompressed_file_handle) ne 'mailbox';
 
-      my $endline = _GET_ENDLINE(\$decompressed_file_handle);
-
-      return ($decompressed_file_handle,$file_type,0,undef,$endline);
+      return ($decompressed_file_handle,$file_type,0,undef);
     }
     else
     {
       dprint "Filehandle is not compressed";
 
-      return ($file_handle,$file_type,0,"No data on filehandle",undef)
-        if eof($file_handle);
+      return ($file_handle,$file_type,0,"No data on filehandle")
+        unless _DATA_ON_FILE_HANDLE($file_handle);
 
-      my $endline = _GET_ENDLINE(\$file_handle);
-
-      return ($file_handle,$file_type,0,"Not a mailbox",$endline)
+      return ($file_handle,$file_type,0,"Not a mailbox")
         if $file_type ne 'mailbox';
 
-      return ($file_handle,$file_type,0,undef,$endline);
+      return ($file_handle,$file_type,0,undef);
     }
   }
   else
   {
-    my $file_type = _GET_FILE_TYPE(\$file_name);
+    my $file_type = _GET_FILE_TYPE($file_name);
     dprint "Filename \"$file_name\" file type: $file_type";
 
     my ($opened_file_handle,$error) =
       _OPEN_FILE_HANDLE($file_name, $file_type);
 
-    return ($file_handle,$file_type,0,$error,undef)
+    return ($file_handle,$file_type,0,$error)
       unless defined $opened_file_handle;
 
-    my $endline = _GET_ENDLINE(\$opened_file_handle);
-
     if (_IS_COMPRESSED_TYPE($file_type))
     {
-      return ($opened_file_handle,$file_type,1,"Not a mailbox",$endline)
-        if _GET_FILE_TYPE(\$opened_file_handle) ne 'mailbox';
+      return ($opened_file_handle,$file_type,1,"Not a mailbox")
+        if _GET_FILE_TYPE($opened_file_handle) ne 'mailbox';
 
-      return ($opened_file_handle,$file_type,1,undef,$endline);
+      return ($opened_file_handle,$file_type,1,undef);
     }
     else
     {
-      return ($opened_file_handle,$file_type,1,"Not a mailbox",$endline)
+      return ($opened_file_handle,$file_type,1,"Not a mailbox")
         if $file_type ne 'mailbox';
 
-      return ($opened_file_handle,$file_type,1,undef,$endline);
+      return ($opened_file_handle,$file_type,1,undef);
     }
   }
 }
@@ -284,11 +253,9 @@ sub _OPEN_FILE_HANDLE
   # Non-compressed file
   unless (_IS_COMPRESSED_TYPE($file_type))
   {
-    my $file_handle = new FileHandle::Unget($file_name);
+    my $file_handle = new FileHandle($file_name);
     return (undef,"Can't open $file_name: $!") unless defined $file_handle;
 
-    binmode $file_handle;
-
     dprint "File \"$file_name\" is not compressed";
 
     return ($file_handle,undef);
@@ -300,23 +267,21 @@ sub _OPEN_FILE_HANDLE
 
   my $filter_command = "$PROGRAMS{$file_type} -cd '$file_name' |";
 
-  dprint "Calling \"$filter_command\" to decompress file \"$file_name\".";
+  dprint "Calling \"$filter_command\" to decompress file. \"$file_name\"";
 
   use vars qw(*OLDSTDERR);
   open OLDSTDERR,">&STDERR" or die "Can't save STDERR: $!\n";
-  open STDERR,">" . File::Spec->devnull()
-    or die "Can't redirect STDERR to " . File::Spec->devnull() . ": $!\n";
+  open STDERR,">/dev/null"
+    or die "Can't redirect STDERR to /dev/null: $!\n";
 
-  my $file_handle = new FileHandle::Unget($filter_command);
+  my $file_handle = new FileHandle($filter_command);
+
+  open STDERR,">&OLDSTDERR" or die "Can't restore STDERR: $!\n";
 
   return (undef,"Can't execute \"$filter_command\" for file \"$file_name\": $!")
     unless defined $file_handle;
 
-  binmode $file_handle;
-
-  open STDERR,">&OLDSTDERR" or die "Can't restore STDERR: $!\n";
-
-  if (eof($file_handle))
+  unless (_DATA_ON_FILE_HANDLE($file_handle))
   {
     $file_handle->close();
     return (undef,"Can't execute \"$filter_command\" for file \"$file_name\"");
@@ -331,160 +296,70 @@ sub _OPEN_FILE_HANDLE
 # bzip2, gzip, compress
 sub _GET_FILE_TYPE
 {
-  my $file_name_or_handle_ref = shift;
+  my $file_name_or_handle = shift;
 
   # Open the file if we need to
-  my $file_handle_ref;
+  my $file_handle;
   my $need_to_close_filehandle = 0;  
 
-  if (ref $file_name_or_handle_ref eq 'SCALAR')
+  if (ref \$file_name_or_handle eq 'SCALAR')
   {
-    my $temp = new FileHandle::Unget($$file_name_or_handle_ref);
-    return 'unknown' unless defined $temp;
-    $file_handle_ref = \$temp;
+    $file_handle = new FileHandle($file_name_or_handle);
+    return 'unknown' unless defined $file_handle;
 
     $need_to_close_filehandle = 1;
   }
   else
   {
-    $file_handle_ref = $file_name_or_handle_ref;
+    $file_handle = $file_name_or_handle;
   }
 
   
   # Read test characters
-  my $test_chars = '';
-  my $readResult;
+  my $testChars;
 
-  while(index($test_chars,"\n\n") == -1 && index($test_chars,"\r\n\r\n") == -1)
-  {
-    $readResult =
-      read($$file_handle_ref,$test_chars,4000,CORE::length($test_chars));
+  binmode $file_handle;
 
-    last unless defined $readResult && $readResult != 0;
+  my $readResult = read($file_handle,$testChars,2000);
 
-    last if _IS_BINARY(\$test_chars);
-
-    if(CORE::length($test_chars) > $MAX_TESTCHAR_BUFFER_SIZE)
-    {
-      if(index($test_chars,"\n\n") == -1 && index($test_chars,"\r\n\r\n") == -1)
-      {
-        dprint "Couldn't find end of first paragraph after " .
-          "$MAX_TESTCHAR_BUFFER_SIZE bytes."
-      }
-
-      last;
-    }
-  }
-
-
-  if($need_to_close_filehandle)
-  {
-    $$file_handle_ref->close();
-  }
-  else
-  {
-    $$file_handle_ref->ungets($test_chars);
-  }
+  $file_handle->close() if $need_to_close_filehandle;
 
   return 'unknown' unless defined $readResult && $readResult != 0;
 
 
-  unless (_IS_BINARY(\$test_chars))
+  _PUT_BACK_STRING($file_handle,$testChars) unless $need_to_close_filehandle;
+
+  # Do -B on the data stream
+  my $isBinary = 0;
   {
-    return 'mailbox' if _IS_MAILBOX(\$test_chars);
+    my $data_length = CORE::length($testChars);
+    my $bin_length = $testChars =~ tr/[\t\n\x20-\x7e]//c;
+    my $non_bin_length = $data_length - $bin_length;
+    $isBinary = ($non_bin_length / $data_length) > .70 ? 0 : 1;
+  }
+
+  unless ($isBinary)
+  {
+    return 'mailbox' if _IS_MAILBOX($testChars);
     return 'non-mailbox ascii';
   }
 
   # See "magic" on unix systems for details on how to identify file types
-  return 'tzip' if substr($test_chars, 0, 2) eq 'TZ';
-  return 'bzip2' if substr($test_chars, 0, 3) eq 'BZh';
-  return 'bzip' if substr($test_chars, 0, 2) eq 'BZ';
-#  return 'zip' if substr($test_chars, 0, 2) eq 'PK' &&
-#    ord(substr($test_chars,3,1)) == 0003 && ord(substr($test_chars,4,1)) == 0004;
+  return 'tzip' if substr($testChars, 0, 2) eq 'TZ';
+  return 'bzip2' if substr($testChars, 0, 3) eq 'BZh';
+  return 'bzip' if substr($testChars, 0, 2) eq 'BZ';
+#  return 'zip' if substr($testChars, 0, 2) eq 'PK' &&
+#    ord(substr($testChars,3,1)) == 0003 && ord(substr($testChars,4,1)) == 0004;
   return 'gzip' if
-    ord(substr($test_chars,0,1)) == 0037 && ord(substr($test_chars,1,1)) == 0213;
+    ord(substr($testChars,0,1)) == 0037 && ord(substr($testChars,1,1)) == 0213;
   return 'compress' if
-    ord(substr($test_chars,0,1)) == 0037 && ord(substr($test_chars,1,1)) == 0235;
+    ord(substr($testChars,0,1)) == 0037 && ord(substr($testChars,1,1)) == 0235;
 
   return 'unknown binary';
 }
 
 #-------------------------------------------------------------------------------
 
-# Returns: undef, "\r\n", "\n"
-sub _GET_ENDLINE
-{
-  my $file_name_or_handle_ref = shift;
-
-  # Open the file if we need to
-  my $file_handle_ref;
-  my $need_to_close_filehandle = 0;  
-
-  if (ref $file_name_or_handle_ref eq 'SCALAR')
-  {
-    my $temp = new FileHandle::Unget($$file_name_or_handle_ref);
-    return 'unknown' unless defined $temp;
-    $file_handle_ref = \$temp;
-
-    $need_to_close_filehandle = 1;
-  }
-  else
-  {
-    $file_handle_ref = $file_name_or_handle_ref;
-  }
-
-  
-  # Read test characters
-  my $test_chars = '';
-  my $readResult;
-
-  while(index($test_chars,"\n") == -1 && index($test_chars,"\r\n") == -1)
-  {
-    $readResult =
-      read($$file_handle_ref,$test_chars,4000,CORE::length($test_chars));
-
-    last unless defined $readResult && $readResult != 0;
-
-    last if _IS_BINARY(\$test_chars);
-
-    if(CORE::length($test_chars) > $MAX_TESTCHAR_BUFFER_SIZE)
-    {
-      if(index($test_chars,"\n") == -1 && index($test_chars,"\r\n") == -1)
-      {
-        dprint "Couldn't find end of first line after " .
-          "$MAX_TESTCHAR_BUFFER_SIZE bytes."
-      }
-
-      last;
-    }
-  }
-
-
-  if($need_to_close_filehandle)
-  {
-    $$file_handle_ref->close();
-  }
-  else
-  {
-    $$file_handle_ref->ungets($test_chars);
-  }
-
-  return undef unless defined $readResult && $readResult != 0;
-
-  return undef if _IS_BINARY(\$test_chars);
-
-  if(index($test_chars,"\r\n") != -1)
-  {
-    return "\r\n";
-  }
-  else
-  {
-    return "\n";
-  }
-}
-
-#-------------------------------------------------------------------------------
-
 sub _IS_COMPRESSED_TYPE
 {
   my $file_type = shift;
@@ -499,68 +374,7 @@ sub _IS_COMPRESSED_TYPE
 
 #-------------------------------------------------------------------------------
 
-# man perlfork for details
-# simulate open(FOO, "-|")
-sub pipe_from_fork ($)
-{
-  my $parent = shift;
-  my $child = new FileHandle::Unget;
-
-  pipe $parent, $child or die;
-
-  my $pid = fork();
-  return undef unless defined $pid;
-
-  if ($pid)
-  {
-      close $child;
-  }
-  else
-  {
-      close $parent;
-      open(STDOUT, ">&=" . fileno($child)) or die;
-  }
-
-  return $pid;
-}
-
-#-------------------------------------------------------------------------------
-
-sub _DO_WINDOWS_DECOMPRESSION
-{
-  my $file_handle = shift;
-  my $file_type = shift;
-
-  return (undef,"Can't decompress file handle--no decompressor available")
-    unless defined $PROGRAMS{$file_type};
-
-  my $filter_command = "$PROGRAMS{$file_type} -cd";
-
-  my ($temp_file_handle, $temp_file_name) =
-    File::Temp::tempfile('mail-mbox-messageparser-XXXXXX', SUFFIX => '.tmp', UNLINK => 1);
-
-  while(my $line = <$file_handle>)
-  {
-    print $temp_file_handle $line;
-  }
-
-  close $file_handle;
-  # So that it won't be deleted until the program is complete
-  # close $temp_file_handle;
-
-  dprint "Calling \"$filter_command\" to decompress filehandle";
-
-  my $decompressed_file_handle =
-    new FileHandle::Unget("$filter_command $temp_file_name |");
-
-  binmode $decompressed_file_handle;
-
-  return ($decompressed_file_handle,undef);
-}
-
-#-------------------------------------------------------------------------------
-
-sub _DO_NONWINDOWS_DECOMPRESSION
+sub _DO_DECOMPRESSION
 {
   my $file_handle = shift;
   my $file_type = shift;
@@ -573,8 +387,8 @@ sub _DO_NONWINDOWS_DECOMPRESSION
   dprint "Calling \"$filter_command\" to decompress filehandle";
 
   # Implicit fork
-  my $decompressed_file_handle = new FileHandle::Unget;
-  my $pid = pipe_from_fork($decompressed_file_handle);
+  my $decompressed_file_handle = new FileHandle;
+  my $pid = $decompressed_file_handle->open('-|');
 
   unless (defined($pid))
   {
@@ -589,11 +403,9 @@ sub _DO_NONWINDOWS_DECOMPRESSION
   # uncompressed input.
   unless ($pid)
   {
-    open(FRONT_OF_PIPE, "|$filter_command 2>" . File::Spec->devnull())
+    open(FRONT_OF_PIPE, "|$filter_command 2>/dev/null")
       or return (undef,"Can't execute \"$filter_command\" on file handle: $!");
 
-    binmode FRONT_OF_PIPE;
-
     print FRONT_OF_PIPE <$file_handle>;
 
     $file_handle->close()
@@ -607,45 +419,39 @@ sub _DO_NONWINDOWS_DECOMPRESSION
     exit;
   }
 
-  binmode $decompressed_file_handle;
-
   # In parent
   return ($decompressed_file_handle,undef);
 }
 
 #-------------------------------------------------------------------------------
 
-sub _DO_DECOMPRESSION
+# Checks to see if there is data on a filehandle, without reading that data.
+
+sub _DATA_ON_FILE_HANDLE
 {
   my $file_handle = shift;
-  my $file_type = shift;
 
-  if ($^O eq 'MSWin32')
-  {
-    return _DO_WINDOWS_DECOMPRESSION($file_handle,$file_type);
-  }
-  else
-  {
-    return _DO_NONWINDOWS_DECOMPRESSION($file_handle,$file_type);
-  }
+  my $buffer = <$file_handle>;
+
+  return 0 unless defined $buffer;
+
+  _PUT_BACK_STRING($file_handle,$buffer);
+
+  return $buffer ? 1 : 0;
 }
 
 #-------------------------------------------------------------------------------
 
-# Simulates -B, which consumes data on a stream.
-sub _IS_BINARY
-{
-  my $data_length = CORE::length(${$_[0]});
-  my $bin_length = ${$_[0]} =~ tr/[\t\n\x20-\x7e]//c;
-  my $non_bin_length = $data_length - $bin_length;
+# Puts a string back on a file handle
 
-  if (($non_bin_length / $data_length) > .70)
+sub _PUT_BACK_STRING
+{
+  my $file_handle = shift;
+  my $string = shift;
+
+  for (my $char_position=CORE::length($string)-1;$char_position >=0; $char_position--)
   {
-    return 0;
-  }
-  else
-  {
-    return 1;
+    $file_handle->ungetc(ord(substr($string,$char_position,1)));
   }
 }
 
@@ -660,8 +466,10 @@ sub _IS_MAILBOX
 {
   my $test_characters = shift;
 
-  if ($$test_characters =~ /$FROM_PATTERN/im &&
-      $$test_characters =~ /^(Received[ :]|Date:|Subject:|X-Status:|Status:|To:)/sm)
+  # X-From-Line is used by Gnus, and From is used by normal Unix
+  # format. Newer versions of Gnus use X-Draft-From
+  if ($test_characters =~ /$FROM_PATTERN/im &&
+      $test_characters =~ /^Received:.*\bfrom\b.*\bby\b.*for\b/sm)
   {
     return 1;
   }
@@ -680,15 +488,6 @@ sub reset
 
 #-------------------------------------------------------------------------------
 
-sub endline
-{
-  my $self = shift;
-
-  return $self->{'endline'};
-}
-
-#-------------------------------------------------------------------------------
-
 sub prologue
 {
   my $self = shift;
@@ -706,19 +505,9 @@ sub _print_debug_information
 
   dprint "Version: $VERSION";
 
-  foreach my $key (keys %$self)
+  foreach my $key ($self)
   {
-    my $value = $self->{$key};
-    if (defined $value)
-    {
-      $value = '<non-scalar>' unless ref \$value eq 'SCALAR';
-    }
-    else
-    {
-      $value = '<undef>';
-    }
-
-    dprint "$key: $value";
+    dprint "$key: $self->{$key}" if ref \$self->{$key} eq 'SCALAR';
   }
 }
 
@@ -840,10 +629,10 @@ Mail::Mbox::MessageParser - A fast and simple mbox folder reader
   print $prologue;
 
   # This is the main loop. It's executed once for each email
-  while(!$folder_reader->end_of_file())
+  while(!$folder_reader->end_of_file());
   {
     my $email = $folder_reader->read_next_email();
-    print $$email;
+    print $email;
   }
 
 =head1 DESCRIPTION
@@ -935,11 +724,6 @@ ability to store the previously read stream data internally so that I<reset()>
 will work correctly.
 
 
-=item endline()
-
-Returns "\n" or "\r\n", depending on the file format.
-
-
 =item prologue()
 
 Returns any newlines or other content at the start of the mailbox prior to the
