@@ -3,8 +3,7 @@ package Test::Utils;
 use strict;
 use Exporter;
 use Test;
-use FileHandle::Unget;
-use File::Spec::Functions qw(:ALL);
+use FileHandle;
 
 use vars qw( @EXPORT @ISA );
 use Mail::Mbox::MessageParser;
@@ -17,13 +16,12 @@ use Mail::Mbox::MessageParser;
 use vars qw( %PROGRAMS );
 
 %PROGRAMS = (
- 'diff' => '/sw/bin/diff',
- 'grep' => '/usr/bin/grep',
- 'tzip' => undef,
- 'gzip' => '/sw/bin/gzip',
- 'compress' => '/sw/bin/gzip',
- 'bzip' => '/sw/bin/bzip2',
- 'bzip2' => '/sw/bin/bzip2',
+ 'grep' => '/bin/grep',
+ 'tzip' => '/home/david/bin/tzip',
+ 'gzip' => '/bin/gzip',
+ 'compress' => '/bin/gzip',
+ 'bzip' => '/usr/bin/bzip2',
+ 'bzip2' => '/usr/bin/bzip2',
 );
 
 sub CheckDiffs
@@ -53,13 +51,13 @@ sub DoDiff
   my $filename = shift;
   my $output_filename = shift;
 
-  my $diffstring = "$PROGRAMS{'diff'} \"$output_filename\" \"$filename\"";
+  my $diffstring = "diff '$output_filename' '$filename'";
 
-  system "echo $diffstring > \"$output_filename.diff\" ".
-    "2>\"$output_filename.diff.error\"";
+  system "echo $diffstring > '$output_filename.diff' ".
+    "2>'$output_filename.diff.error'";
 
-  system "$diffstring >> \"$output_filename.diff\" ".
-    "2>\"$output_filename.diff.error\"";
+  system "$diffstring >> '$output_filename.diff' ".
+    "2>'$output_filename.diff.error'";
 
   open DIFF_ERR, "$output_filename.diff.error";
   my $diff_err = join '', <DIFF_ERR>;
@@ -81,7 +79,7 @@ sub DoDiff
 
   local $/ = "\n";
 
-  my @diffs = `cat "$output_filename.diff"`;
+  my @diffs = `cat '$output_filename.diff'`;
   shift @diffs;
   my $numdiffs = ($#diffs + 1) / 2;
 
@@ -109,12 +107,10 @@ sub InitializeCache
 {
   my $filename = shift;
 
-  my $cache_file = catfile('t','temp','cache');
-
-  Mail::Mbox::MessageParser::SETUP_CACHE({'file_name' => $cache_file});
+  Mail::Mbox::MessageParser::SETUP_CACHE({'file_name' => 't/temp/cache'});
   Mail::Mbox::MessageParser::CLEAR_CACHE();
 
-  my $filehandle = new FileHandle::Unget($filename);
+  my $filehandle = new FileHandle($filename);
 
   my $folder_reader =
       new Mail::Mbox::MessageParser( {
@@ -145,12 +141,12 @@ sub ModuleInstalled
 {
   my $module_name = shift;
 
-  $module_name =~ s#::#/#g;
+  $module_name =~ s/::/\//g;
   $module_name .= '.pm';
 
   foreach my $inc (@INC)
   {
-    return 1 if -e catfile($inc,$module_name);
+    return 1 if -e "$inc/$module_name";
   }
 
   return 0;
@@ -182,16 +178,13 @@ sub No_such_file_or_directory
 # doing this?
 sub Broken_Pipe
 {
-  mkdir catdir('t','temp'), 0700;
+  mkdir 't/temp', 0700;
 
-  my $script_path = catfile('t','temp','broken_pipe.pl');
-  my $dev_null = devnull();
-
-  open F, ">$script_path";
+  open F, ">t/temp/broken_pipe.pl";
   print F<<EOF;
 unless (open B, '-|')
 {
-  open(F, "|cat 2>$dev_null");
+  open(F, "|cat 2>/dev/null");
   print F 'x';
   close F;
   exit;
@@ -199,7 +192,7 @@ unless (open B, '-|')
 EOF
   close F;
 
-  my $result = `$^X $script_path 2>&1 1>$dev_null`;
+  my $result = `$^X t/temp/broken_pipe.pl 2>&1 1>/dev/null`;
 
   $result = '' unless defined $result;
 
